!---------------------------------------------------------------------------------------
! PREP_SATELLITE: prep_command
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> prep_command
!!
!! Purpose:    Prepares the options file COMMAND.
!!
!! Interface:
!!
!!    Inputs
!!             settings  -  settings data structure
!!             jd        -  julian day for start of month
!!
!!    Externals
!!             caldate
!!             juldate
!!
!---------------------------------------------------------------------------------------

subroutine prep_command(settings, jd)

  use mod_var
  use mod_settings
  use mod_dates

  implicit none

  type (settings_t), intent(in) :: settings
  real(kind=8),      intent(in) :: jd

  character(len=max_path_len)   :: filename
  character(len=8)              :: adate
  integer                       :: jjjjmm, jjjjmmdd, hhmiss
  integer                       :: datei, datef
  integer                       :: ierr

  integer                       :: ldirect                ! runtime mode (currently only backwards: ldirect = -1)
  integer                       :: ibdate, ibtime
  integer                       :: iedate, ietime
  integer                       :: partsplit
  integer                       :: loutstep
  integer                       :: loutaver
  integer                       :: loutsample
  integer                       :: itsplit                ! time constant for particle splitting (secs)
  integer                       :: lsynctime              ! synchronisation interval of flexpart (secs)
  real                          :: ctl                    ! factor by which time step must be smaller than tl
  integer                       :: ifine                  ! factor by which to decrease time step for vertical motion
  integer                       :: iout                   ! output type (1 = concentration/residence time, 2 = mixing ratio, 3 = both, 4 = plume trajectory)
  integer                       :: ipout                  ! particle dump (0 = never, 1 = every output interval, 2 = only at end)
  integer                       :: lsubgrid               ! use subgrid terrain effect parameterization (0 = no, 1 = yes)
  integer                       :: lconvection            ! use convection (0 = no, 1 = yes)
  integer                       :: lagespectra            ! calculate age spectra (0 = no, 1 = yes)
  integer                       :: ipin                   ! continue simulation with dumped particle data (0 = no, 1 = yes)
  integer                       :: ioutputforeachrelease  ! create output file for each release location (0 = no, 1 = yes)
  integer                       :: mdomainfill            ! domain filling option (0 = no, 1 = yes)
  integer                       :: mquasilag              ! quasilagrangian mode to track particles (0 = no, 1 = yes)
  integer                       :: surf_only              ! save only surface layer in grid_time files = 1, full resolution = 0
  integer                       :: lnetcdfout             ! netcdf output (0 = no, 1 = yes)
  integer                       :: cblflag                ! 
  integer                       :: linversionout          ! one grid_time file per release = 1, or per timestep (original format) = 0
  integer                       :: satellite_releases     ! releases for satellite retrievals 
  integer                       :: iflux                  ! calculate fluxes (0 = no, 1 = yes)
  integer                       :: ind_source
  integer                       :: ind_receptor
  integer                       :: nested_output
  integer                       :: linit_cond
  character(len=max_path_len)   :: ohfields_path

  namelist /command/ &
    ldirect, &
    ibdate,ibtime, &
    iedate,ietime, &
    loutstep, &
    loutaver, &
    loutsample, &
    itsplit, &
    lsynctime, &
    ctl, &
    ifine, &
    iout, &
    ipout, &
    lsubgrid, &
    lconvection, &
    lagespectra, &
    ipin, &
    ioutputforeachrelease, &
    iflux, &
    mdomainfill, &
    ind_source, &
    ind_receptor, &
    mquasilag, &
    nested_output, &
    linit_cond, &
    lnetcdfout, &
    surf_only, &
    cblflag, &
    linversionout, &
    satellite_releases, &
    ohfields_path

  ! initialize command file path and name
  call caldate(jd, jjjjmmdd, hhmiss)
  write(adate,fmt='(I8.8)') jjjjmmdd
  filename = trim(settings%path_options)//adate//'/options/COMMAND'

  call system('mkdir -p '//trim(settings%path_options)//adate//'/options/')

  ! get dates
  ! correct start date for length of back trajectory
  call caldate((jd-settings%ageclass/3600d0/24d0), jjjjmmdd, hhmiss)
  datei = jjjjmmdd
  call caldate(jd+1d0, jjjjmmdd, hhmiss)
  datef = jjjjmmdd

  ! preset namelist variables
  ldirect = -1
  ibdate = datei
  ibtime = 0
  iedate = datef
  ietime = 0
  loutstep = settings%outrate
  loutaver = settings%outaverage
  loutsample = settings%outsample
  itsplit = 999999999
  lsynctime = 600
  ctl = -5
  ifine = 4
  iout = 1
  ipout = 2
  lsubgrid = 1
  lconvection = 1
  lagespectra = 1
  ipin = 0
  ioutputforeachrelease = 1
  iflux = 0
  mdomainfill = 0
  ind_source = settings%ind_source
  ind_receptor = settings%ind_receptor
  mquasilag = 0
  nested_output = settings%lnested
  linit_cond = settings%linit_cond
  lnetcdfout = 0
  surf_only = 1
  cblflag = 0
  linversionout = 1
  satellite_releases = 1
  ohfields_path = trim(settings%path_ohfield)

  ! write command file
  open(100,file=trim(filename),status='replace',action='write',iostat=ierr)    
  write(100,nml=command)
  close(100)

end subroutine prep_command

