!---------------------------------------------------------------------------------------
! PREP_SATELLITE: prep_outgrid
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> prep_outgrid
!!
!! Purpose:    Prepares the options file OUTGRID
!!
!! Interface:
!!
!!    Inputs
!!             settings  -  settings data structure
!!             jd        -  julian day for start of month
!!
!!    Externals
!!             caldate
!!             skiplines
!!
!---------------------------------------------------------------------------------------

subroutine prep_outgrid(settings, jd)

  use mod_var
  use mod_settings
  use mod_dates

  implicit none

  type (settings_t), intent(in)      :: settings
  real(kind=8),      intent(in)      :: jd

  character(len=max_path_len)        :: filename
  character(len=8)                   :: adate
  integer                            :: jjjjmmdd, hhmiss
  integer                            :: ierr, n

  real                               :: outlon0, outlat0, outlon0n, outlat0n
  integer                            :: numxgrid, numygrid, numxgridn, numygridn
  real                               :: dxout, dyout, dxoutn, dyoutn
  real, dimension(settings%numzgrid) :: outheights

  namelist /outgrid/ &
    outlon0, &
    outlat0, &
    numxgrid, &
    numygrid, &
    dxout, &
    dyout, &
    outheights

  namelist /outgridn/ &
    outlon0n, &
    outlat0n, &
    numxgridn, &
    numygridn, &
    dxoutn, &
    dyoutn

  ! write OUTGRID

  ! preset namelist variables
  outlon0 = settings%outlonleft
  ! correction for FLEXPART windfields
  if ( settings%outlonleft.le.-180. ) then
    if ( settings%windfield.eq.'oper' ) then
      outlon0 = -179.
    else if ( settings%windfield.eq.'era5' ) then
      outlon0 = -179.5
    else
      write(logid,*) 'ERROR: prep_outgrid: unknown windfield type'
      stop
    endif
  endif
  outlat0 = settings%outlatlower
  numxgrid = settings%numxgrid
  numygrid = settings%numygrid
  dxout = settings%dxout
  dyout = settings%dyout
  do n = 1, settings%numzgrid
    outheights(n) = settings%zlevel(n)
  end do

  call caldate(jd, jjjjmmdd, hhmiss)
  write(adate,fmt='(I8.8)') jjjjmmdd
  filename = trim(settings%path_options)//adate//'/options/OUTGRID'

  ! write namelist file format
  open(100,file=trim(filename),status='replace',action='write',iostat=ierr)
  write(100,nml=outgrid)
  close(100)
  
  ! write OUTGRID_NEST

  if( settings%lnested.eq.1 ) then

    ! preset namelist variables
    outlon0n = settings%outlonnest
    outlat0n = settings%outlatnest
    numxgridn = settings%numxnest
    numygridn = settings%numynest
    dxoutn = settings%dxoutnest
    dyoutn = settings%dyoutnest

    call caldate(jd, jjjjmmdd, hhmiss)
    write(adate,fmt='(I8.8)') jjjjmmdd
    filename = trim(settings%path_options)//adate//'/options/OUTGRID_NEST'

    ! write namelist file format
    open(100,file=trim(filename),status='replace',action='write',iostat=ierr)
    write(100,nml=outgridn)
    close(100)    
  
  endif

end subroutine prep_outgrid

