!---------------------------------------------------------------------------------------
! PREP_SYNDATA: initialize
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> initialize
!!
!! Purpose:    Initializes variables
!!
!---------------------------------------------------------------------------------------

subroutine initialize(files, config)

  use mod_var
  use mod_settings
  use mod_dates
  use mod_tools
  use mod_flexpart, only : read_header

  implicit none

  type (files_t),  intent(in out)        :: files
  type (config_t), intent(in)            :: config
  
  character(max_path_len)                :: filename
  character(max_path_len)                :: rowfmt
  logical                                :: lexist
  integer                                :: yyyymm, yyyymmdd, hhmiss
  character(6)                           :: adate
  character(8)                           :: areldate
  character(4)                           :: ayear
  integer                                :: numpoint
  integer                                :: ibdate, ibtime
  integer, dimension(maxpoint)           :: releases
  real                                   :: area
  integer                                :: i, ix, jy
  real                                   :: bndx, bndy, delx, dely
  integer                                :: numx, numy, numz, xshift
  integer                                :: ierr

  ! open log file
  open(logid,file=trim(files%path_output)//'prep_syndata.log',status='replace',action='write',iostat=ierr)

  ! read receptor list file
  ! -----------------------

  filename = trim(files%file_recept)
  inquire(file=trim(filename),exist=lexist)
  if ( .not.lexist.and.config%ground ) then
    write(logid,*) 'ERROR: cannot find '//trim(filename)
    stop
  else if ( lexist.and.config%ground ) then
    write(logid,*) 'Reading receptor list: '//trim(filename)
    call read_reclist(filename)
  endif

  ! initialize flexpart variables
  ! -----------------------------

  ! read header 
  if ( config%satellite.and.nrec.eq.0 ) then
    ! satellite only
    write(areldate,'(I8)') config%datei
    filename = trim(files%path_flexsat)//areldate//'/header'
    inquire(file=trim(filename),exist=lexist)
  else
    ! read from ground-based output
    yyyymm=config%datei/100
    write(adate,'(i6)') yyyymm
    lexist = .false.
    i = 1
    do while ( (.not.lexist).and.(i.le.nrec) )
      if ( .not.config%nested ) then
        filename = trim(files%path_flexpart)//trim(recname(i))//'/'//adate//'/header'
        inquire(file=trim(filename),exist=lexist)
      else
        filename = trim(files%path_flexpart)//trim(recname(i))//'/'//adate//'/header_nest'
        inquire(file=trim(filename),exist=lexist)
        if ( .not.lexist ) then
          ! try if is FPv11 file
          filename = trim(files%path_flexpart)//trim(recname(i))//'/'//adate//'/header_nest_grid_time'
          inquire(file=trim(filename),exist=lexist)
        endif
      endif
      i = i + 1
    end do
  endif ! satellite
  if ( .not.lexist ) then
    write(logid,*) 'ERROR initialize: cannot find flexpart header'
    stop
  endif
  write(logid,*) 'Reading flexpart header: '//trim(filename)
  call read_header(filename, numpoint, ibdate, ibtime, releases, &
                     numx, numy, numz, bndx, bndy, delx, dely, xshift)
  ! global domain variables
  nxgrid = numx
  nygrid = numy
  llx = bndx
  lly = bndy
  dx = delx
  dy = dely
  nxshift = xshift
  if ( .not.allocated(gbl_lon) ) allocate( gbl_lon(nxgrid) )
  if ( .not.allocated(gbl_lat) ) allocate( gbl_lat(nygrid) )
  do i = 1, nxgrid
    gbl_lon(i) = llx + (i-1)*dx
  end do
  do i = 1, nygrid
    gbl_lat(i) = lly + (i-1)*dy
  end do

  ! initialize date variables
  ! -------------------------

  ! julian dates for inversion interval
  juldatei = juldate(config%datei, 0)
  juldatef = juldate(config%datef, 0)

  ! number days in inversion interval
  nday = int(juldatef - juldatei + 1.)
  write(logid,*) 'initialize: nday = ',nday

  ! timestep of fluxes (hours)
  nt_nee = config%nstep_nee
  nt_flx = config%nstep_flx

  ! number of NEE fields per day
  if ( config%spec.eq.'co2' ) then
    if ( nt_nee.eq.0 ) then
      write(logid,*) 'ERROR: nt_nee cannot be zero'
      stop
    endif
    nd_nee = 24/nt_nee
  endif

  if( config%spec.eq.'co2' ) then
    ! statres is time interval over which state variables are averaged
    ! statres_hr is the sub-daily time interval for state variables
    ! ndt is number of time steps per 24h in prior NEE
    ! ntstate is number of averaged time invervals for state variables
    ! nflx is number of prior flux fields in inversion interval
    statres = config%statres
    statres_hr = config%statres_hr
    ndt = int(24./statres_hr)
    ntstate = int(real(nday)/statres)
  else
    ! statres is time resolution of state variables
    ! ntstate is also number of state time invervals 
    ! nflx is number of prior flux fields in inversion interval
    statres = config%statres
    ntstate = int(real(nday)/statres)
    ndt = 1
  endif

  ! adjust nday to match number of state time steps
  nday = ntstate*int(statres)
  write(logid,*) 'initialize: adjusted nday to match number of state time steps = ',nday
  juldatef = juldatei + dble(nday) - 1d0
  call caldate(juldatef, yyyymmdd, hhmiss)
  write(logid,*) 'initialize: adjusted end date to match number of state time steps = ',yyyymmdd

  if ( config%spec.eq.'co2' ) then
    nflx = nday*nd_nee
  else
    nflx = nday*24/nt_flx
  endif

  if ( ntstate.lt.1 ) then
    write(logid,*) 'ERROR initialize: number flux time steps < 1 -> increase time interval'
    stop
  endif
  if ( ntstate.lt.1 ) then
    write(logid,*) 'ERROR initialize: state time steps < 1 -> increase time interval'
    stop
  endif

  write(logid,*) 'statres: ',statres
  write(logid,*) 'flxtres: ',flxtres
  write(logid,*) 'ntstate: ',ntstate
  write(logid,*) 'nt_flx: ',nt_flx
  write(logid,*) 'nflx: ', nflx
  write(logid,*) 'nt_nee: ',nt_nee
  write(logid,*) 'nd_nee: ',nd_nee
  write(logid,*) 'nday: ',nday
  write(logid,*) 'nyr: ',nyr

  ! initialize grid variables
  ! -------------------------
 
  rllx = config%w_edge_lon
  rlly = config%s_edge_lat
  rurx = config%e_edge_lon
  rury = config%n_edge_lat
  rdx = config%xres
  rdy = config%yres
  nxregrid = int((rurx - rllx)/rdx) 
  nyregrid = int((rury - rlly)/rdy)

  allocate( reg_lon(nxregrid) )
  allocate( reg_lat(nyregrid) )

  do ix = 1, nxregrid
    reg_lon(ix) = rllx + (ix-1)*rdx
  end do
  do jy = 1, nyregrid
    reg_lat(jy) = rlly + (jy-1)*rdy
  end do

  ! grid operator
  ! -------------

  allocate( nbox_xy(nxregrid,nyregrid) )
  filename = trim(files%path_output)//'nbox_xy.txt'
  inquire(file=filename, exist=lexist)
  if ( .not.lexist ) then
    write(logid,*) 'ERROR: cannot find '//trim(filename)
    stop
  endif
  open(100, file=filename, action='read', status='old')
  write(rowfmt,'(A,I6,A)') '(',nyregrid,'(I5,1X))'
  do ix = 1, nxregrid
    read(100,fmt=rowfmt) nbox_xy(ix,:)
  end do

  nbox = maxval(nbox_xy)

  allocate( area_box(nbox) )
  area_box(:) = 0.
  do jy = 1, nyregrid
    do ix = 1, nxregrid
      area = grid_area(reg_lat(jy)+0.5*rdy, rdy, rdx)
      if ( nbox_xy(ix,jy).gt.0 ) then
        area_box(nbox_xy(ix,jy)) = area_box(nbox_xy(ix,jy)) + area
      endif
    end do
  end do 

  ! local time
  ! ----------

  if ( config%spec.eq.'co2' ) then
    ! read local-time to UTC offsets for boxes
    filename = trim(files%path_output)//'hloc_box.txt'
    inquire(file=filename, exist=lexist)
    if ( .not.lexist ) then
      write(logid,*) 'ERROR: cannot find '//trim(filename)
      stop
    endif
    allocate( hloc(nbox) )
    open(100,file=filename,action='read',status='old')
    do i = 1, nbox
      read(100,*) hloc(i)
    end do
    close(100) 
  endif

  if ( config%spec.eq.'co2' ) then
    nvar = nbox*ntstate*ndt
    ndvar = nbox*ndt
  else
    nvar = nbox*ntstate
    ndvar = nbox
  endif

  write(logid,*) 'nvar = ',nvar
  write(logid,*) 'ndvar = ',ndvar

end subroutine initialize


