!---------------------------------------------------------------------------------------
! FLEXINVERT: main
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> main
!!
!! Purpose:    Main routine of FLEXINVERT
!!
!! References: Thompson R.L. and Stohl, A., Geosci. Model Devel., 2014
!!     
!---------------------------------------------------------------------------------------

program main

  use mod_var
  use mod_settings
  use mod_regions
  use mod_fluxes
  use mod_states
  use mod_covar
  use mod_obs
  use mod_save

  implicit none

  ! set types
  type (files_t)           :: files
  type (config_t)          :: config
  type (fluxes_t)          :: fluxes
  type (states_t)          :: states
  type (covar_t)           :: covar
  type (obs_t)             :: obs
  
  character(max_path_len)  :: settings_files
  character(max_path_len)  :: settings_config

  ! initialization
  ! --------------

  call getarg(1,settings_files)
  if ( settings_files == '' ) then
    stop 'ERROR: need to specify SETTINGS_files'
  endif
 
  call getarg(2,settings_config)
  if ( settings_config == '' ) then
    stop 'ERROR: need to specify SETTINGS_config'
  endif

  ! read file settings
  call read_file_settings(settings_files, files)

  ! read config settings
  call read_config_settings(settings_config, config)

  ! copy settings files to output path
  call system('cp '//trim(settings_files)//' '//trim(files%path_output))
  call system('cp '//trim(settings_config)//' '//trim(files%path_output))

  ! initialize parameters
  call initialize(files, config)

  ! read orography
  call read_orog(files)

  ! read land-sea mask 
  call read_lsm(files)

  ! define grid boxes or regions
  call get_regions(files, config)

  ! prior and observations
  ! -----------------------

  if ( config%spec.eq.'co2' ) then
    call init_co2(files, config, fluxes, obs, states, covar)
  else if ( config%spec.eq.'ghg' ) then
    call init_ghg(files, config, fluxes, obs, states, covar)
  else if ( config%spec.eq.'aer' ) then
    call init_aero(files, config, fluxes, obs, states, covar)
  else
    write(logid,*) 'ERROR: spec must be one of co2, ghg, aer'
    stop
  endif

  ! average footprints if applicable
  ! --------------------------------

  ! averages footprints over multiple releases to match an observation
  ! averaged footprints are saved as ncdf files for later use 
  if ( config%average_fp ) then
    call average_fp(files, config, obs)
  endif
 
  ! optimization
  ! ------------

  call retrieval(files, config, fluxes, obs, states, covar)

  ! save output
  ! -----------

  call save_obs(files, config, obs)
  call save_state(files, config, fluxes, states)

  ! prepare to exit
  ! ---------------

  if ( allocated(nbox_xy) )  deallocate(nbox_xy)
  if ( allocated(lsm_box) )  deallocate(lsm_box)
  if ( allocated(gbl_lon) )  deallocate(gbl_lon)
  if ( allocated(gbl_lat) )  deallocate(gbl_lat)
  if ( allocated(reg_lon) )  deallocate(reg_lon)
  if ( allocated(reg_lat) )  deallocate(reg_lat)
  if ( allocated(lsm) )      deallocate(lsm)
  if ( allocated(hloc) )     deallocate(hloc)
  if ( allocated(area_box) ) deallocate(area_box)
  call dealloc_covar(covar)
  call dealloc_fluxes(fluxes)
  call dealloc_states(states)
  call dealloc_obs(obs)
  close(logid)

end program



